@@ -1,4 +1,5 @@
 use plotpy::{Curve, Plot};
+use russell_lab::Vector;
 use std::fs::File;
 use std::io::{BufRead, BufReader};
 use std::path::Path;
@@ -17,11 +18,12 @@ fn test_plot() -> Result<(), &'static str> {
 
     // configure plot
     let mut plot = Plot::new();
+    plot.set_figure_size(400.0, 0.5);
     plot.set_subplot(2, 2, 1)
         .set_horizontal_gap(0.1)
         .set_vertical_gap(0.2)
         .set_gaps(0.3, 0.4)
-        .set_equal_axes()
+        .set_equal_axes(true)
         .set_hide_axes(false)
         .set_range(-1.0, 1.0, -1.0, 1.0)
         .set_range_from_vec(&[0.0, 1.0, 0.0, 1.0])
@@ -48,6 +50,9 @@ fn test_plot() -> Result<(), &'static str> {
     // add curve to plot
     plot.add(&curve);
 
+    // extra commands
+    plot.write_extra("plt.plot([2.0,10.0],[3.0,1.0])\n");
+
     // save figure
     let path = Path::new(OUT_DIR).join("integ_plot.svg");
     plot.save(&path)?;
@@ -56,7 +61,7 @@ fn test_plot() -> Result<(), &'static str> {
     let file = File::open(path).map_err(|_| "cannot open file")?;
     let buffered = BufReader::new(file);
     let lines_iter = buffered.lines();
-    assert!(lines_iter.count() > 650);
+    assert!(lines_iter.count() > 550);
     Ok(())
 }
 
@@ -79,10 +84,10 @@ fn test_plot_subplots() -> Result<(), &'static str> {
 
     // configure plot
     let mut plot = Plot::new();
-    plot.set_super_title("all subplots");
-    plot.set_horizontal_gap(0.5);
-    plot.set_vertical_gap(0.5);
-    plot.set_gaps(0.3, 0.2);
+    plot.set_super_title("all subplots")
+        .set_horizontal_gap(0.5)
+        .set_vertical_gap(0.5)
+        .set_gaps(0.3, 0.2);
 
     // add curve to subplots
     plot.set_subplot(2, 2, 1);
@@ -105,3 +110,55 @@ fn test_plot_subplots() -> Result<(), &'static str> {
     assert!(lines_iter.count() > 980);
     Ok(())
 }
+
+#[test]
+fn test_plot_log() -> Result<(), &'static str> {
+    // curves
+    let mut curve1 = Curve::new();
+    let mut curve2 = Curve::new();
+    let mut curve3 = Curve::new();
+    let mut curve4 = Curve::new();
+
+    // draw curve
+    let x = Vector::linspace(1.0, 11.0, 11);
+    let y = x.get_mapped(|v| f64::exp(v));
+    curve1.draw(&x, &x);
+    curve2.draw(&x, &y);
+    curve3.draw(&y, &x);
+    curve4.draw(&y, &y);
+
+    // configure plot
+    let mut plot = Plot::new();
+
+    // add curve to subplots
+    plot.set_subplot(2, 2, 1);
+    plot.set_log_x(false);
+    plot.set_log_y(false);
+    plot.add(&curve1);
+
+    plot.set_subplot(2, 2, 2);
+    plot.set_log_x(false);
+    plot.set_log_y(true);
+    plot.add(&curve2);
+
+    plot.set_subplot(2, 2, 3);
+    plot.set_log_x(true);
+    plot.set_log_y(false);
+    plot.add(&curve3);
+
+    plot.set_subplot(2, 2, 4);
+    plot.set_log_x(true);
+    plot.set_log_y(true);
+    plot.add(&curve4);
+
+    // save figure
+    let path = Path::new(OUT_DIR).join("integ_plot_log.svg");
+    plot.save(&path)?;
+
+    // check number of lines
+    let file = File::open(path).map_err(|_| "cannot open file")?;
+    let buffered = BufReader::new(file);
+    let lines_iter = buffered.lines();
+    assert!(lines_iter.count() > 980);
+    Ok(())
+}
