@@ -37,13 +37,13 @@ fn test_shapes() -> Result<(), &'static str> {
 
     // save figure
     let path = Path::new(OUT_DIR).join("integ_shapes.svg");
-    plot.set_equal_axes();
+    plot.set_equal_axes(true);
     plot.save(&path)?;
 
     // check number of lines
     let file = File::open(path).map_err(|_| "cannot open file")?;
     let buffered = BufReader::new(file);
     let lines_iter = buffered.lines();
-    assert!(lines_iter.count() > 480);
+    assert!(lines_iter.count() > 450);
     Ok(())
 }
